/***************************************************************************
    defs.h  -  all definitions
    -------------------------------------
    Begin                : pia lut 11 09:59:54 CET 2000
    Copyright            : (C) 2000 by Tomasz Fiszer
    Email                : profis@kki.net.pl
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <termios.h>
#include "pslock.h"
#include "struct.h"
// Debug definitions

// Version information and copyright
#define PRIMARY_VER		"1.1.1b"
#define PRIMARY_COPYRIGHT	"Author: Tomasz Fiszer <tfiszer@ever.com.pl> "\
				"EVER Sp. z o.o."


#ifdef _USE_TRACE

#ifdef _TRACE_MSG0
#  define TRACE	{ \
		  printf("Processing line %u in %s\n", __LINE__, __FILE__); \
		  fflush(stdout); \
		}
#  define TRACEF(a){ \
		  printf("Processing line %u in %s\n", __LINE__, __FILE__); \
		  printf a; \
		  fflush(stdout); \
		}
#else /* _TRACE_MSG0 */
#  define TRACE
#  define TRACEF(a)
#endif /* _TRACE_MSG0 */

#ifdef _TRACE_MSG1
#  define TRACEF1(a){ \
		  printf("Processing line %u in %s\n", __LINE__, __FILE__); \
		  printf a; \
		  fflush(stdout); \
		}
#else /* _TRACE_MSG1 */
#  define TRACEF1(a)
#endif /* _TRACE_MSG1 */
#ifdef _TRACE_MSG2
#  define TRACEF2(a){ \
		  printf("Processing line %u in %s\n", __LINE__, __FILE__); \
		  printf a; \
		  fflush(stdout); \
		}
#else /* _TRACE_MSG2 */
#  define TRACEF2(a)
#endif /* _TRACE_MSG2 */

#else /* _USE_TRACE */
#  define TRACE
#  define TRACEF(a)
#  define TRACEF1(a)
#  define TRACEF2(a)
#endif /* _USE_TRACE */

#ifndef _SR_TIMEOUT
#  define _SR_TIMEOUT				25000		// Send receive timeout
#endif /* _SR_TIMEOUT */

#ifndef RECOVERY_TIMEOUT
#  define RECOVERY_TIMEOUT			60
#endif

// ERRORS definition
#define SUCCESS_ONESIDE				200
#define SUCCESS_TWOSIDE				201
#define SUCCESS					0
#define	FAILURE					-1
#define	_ERROR					-1
#define	_ERROR_CONFIG				-2

#define _ERROR_NOCOMM				-10
#define _ERROR_COMM_TIMEOUT			-11
#define	_ERROR_SETUP_SERIAL			-12

#define _SUCCESS_LOCKFILE			0
#define _ERROR_LOCKFILE				-21
#define _ERROR_LOCKFILE_EXIST			-22

#define	_ERROR_CONFIG_ENTRY_NULL		1
#define _ERROR_BAD_PARAM			-100


// STRINGS definition
#define STRING_ERROR_LOAD_CONFIG	"Error occured while opening config file"
#define STRING_ERROR_NOT_ROOT		"I need ROOT privileges!\n"
#define STRING_INIT_FINISHED		"Program initialization success.\n"

#ifndef POWERSOFTCONF
#define POWERSOFTCONF		"/etc/powersoft/powersoft.conf"
#endif

#ifndef UPS_STATUS
#define UPS_STATUS              "/etc/powersoft/powersoft.status"
#endif

#ifndef PWRFAIL
#define PWRFAIL			"/etc/powerfail"
#endif

#ifndef POWERSOFT_HALT
#define POWERSOFT_HALT		"shutdown -h now"
#endif

#define NOLOGIN			"/etc/nologin"
#define POWERSOFTPID		"/var/run/powersoft.pid"
#define LOG_FILE		"/var/log/powersoft.log"
#define NET_LOG_FILE		"psnet.log"
#define POWERSOFT_CONFIG_FILE	"/etc/powersoft/powersoft.conf"

#define LOCK_NAME		"LCK.."
#define LOCK_DEFAULT		"/var/lock"
#define MAXTHREADS		3
#define BAUDRATE		B300

#define PS_FIFO_SERVER		"/tmp/psserver.fifo"
#define PS_FIFO_CLIENT		"/tmp/psclient.fifo"

#define FIFO_AUTOTEST		1
#define FIFO_RECONFIGURE	2

#ifndef TRUE
#define TRUE			1
#endif
#ifndef FALSE
#define FALSE			0
#endif

#define MAIN_LOOP_TIMER		5

// UPS State definitions
// 0	- Online; 
// 1	- Battery;
// 2	- Standby;
// 3	- After UPS error;
// 4>>	- Reserved;
// 7	- Error;
#define ONLINE_WORK		0x00000001
#define BATTERY_WORK		0x00000002
#define STANDBY_WORK		0x00000004
#define AFTER_ERROR		0x00000008
#define ERROR			0x00000010

#define FIRST_TIME_ONLINE	0x00000001
#define FIRST_TIME_BATTERY	0x00000002
#define FIRST_TIME_STANDBY	0x00000004

#define MSG_SHUTDOWN_SEND	0x00000100
#define CMD_SHUTDOWN_SEND	0x00000200

struct termios 	oldtio;
struct termios	newtio;

unsigned char	cSendBuffer;
unsigned char	cReceiveBuffer;

extern int LE_BIT;
extern int DTR_BIT;
extern int RTS_BIT;
extern int ST_BIT;
extern int SR_BIT;
extern int CTS_BIT;
extern int CD_BIT;
extern int RNG_BIT;
extern int DSR_BIT;

extern void Terminate(int);
extern void TerminateThread();
extern int Code(UPS_INFO *);

extern int iThreadPID[MAXTHREADS];
//#endif /* _DEFS_H_ */
