/***************************************************************************
    psnet.c  -  Internet interface for UPS statistics
    ------------------------
    begin	: pia lut 11 09:59:54 CET 2000
    copyright	: (C) 2000 by Tomasz Fiszer
    email	: profis@kki.net.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>

#include "defs.h"
#include "struct.h"
#include "utils.h"
#include "strman.h"
#include "conf.h"
#include "fifo.h"

#define STR_LEN		50

typedef 	unsigned int UINT;
// Global variables definition
extern int 	errno;

// Global Structures		
static UPS_INFO		UpsInfo;
static UPS_INFO_FIFO	UpsInfoFifo;

static int	iOperation = 0;

//
// Global function definitions
//
void	InitAllVariables();
void	PrintUPSInfo(UPS_INFO_FIFO *);
int	ExecuteCommand(int);
void	ConnectToUPS();
char	*Spaces(char *, int);
void	Terminate(int);
void	TerminateThread();
char	*GetTypeUpsName(UPS_INFO_FIFO *);
char	*GetUpsState(UPS_INFO_FIFO *);
char	*GetAccuString(char *, unsigned long);
char	*GetAccuCapacityString(char *, UPS_INFO_FIFO);
char	*GetTimeString(char *, unsigned long);
int	GetAccuCapacity(UPS_INFO_FIFO);
void	MainThread(int);
void	Autotest();
char	*Warning(char *, UPS_INFO_FIFO);
void	PrintHackerInfo();
void	PrintInfo(char *);
void	PrintAutotestResult();

char *Spaces(char *szBuf, int in)
{
    int icnt;
    for (icnt = 0; icnt < in; icnt++)
	szBuf[icnt] = ' ';	// 32(Dec)=space
    szBuf[icnt] = '\0';
    return szBuf;
}

//
// initialization of all variables
//
void InitAllVariables()
{
    memset(&UpsInfo, 0, sizeof(UpsInfo));
}

//
// Retrieve Name of UPS
//
char *GetTypeUpsName(UPS_INFO_FIFO *Ups)
{
TRACE;
    switch(Ups->uiUpsType)
    {
	case 67: return "NET 500-DPC";
	case 68: return "NET 700-DPC";
	case 69: return "NET 1000-DPC";
	case 70: return "NET 1400-DPC";
	case 71: return "NET 2200-DPC";

	case 73: return "NET 700-DPC";
	case 74: return "NET 1000-DPC";
	case 75: return "NET 1400-DPC";
	case 76: return "NET 500-DPC";

	case 81: return "AP 450-PRO";
	case 82: return "AP 650-PRO";
	default: return "Brak danych";
    }
}

//
// returns string about current state of UPS
//
char *GetUpsState(UPS_INFO_FIFO *Ups)
{
TRACE;
    if (IsStatusSet(Ups->ulUpsState, BATTERY_WORK))
	return 		"PRACA BATERYJNA! (Brak zasilania)";
    else
	if (IsStatusSet(Ups->ulUpsState, STANDBY_WORK))
	    return	"TRYB OCZEKIWANIA                 ";
	else
	    return	"PRACA SIECIOWA (Zasilanie OK)    ";
}

//
// Returns accu. voltage string (float type)
//
char *GetAccuString(char *szBuff, unsigned long ulAccuVal)
{
    float fVal;
    fVal = ulAccuVal / 10.0;
    *szBuff = 0;
    sprintf(
	szBuff,
	"%.1f",
	fVal);
    return szBuff;
}

//
// Returns current accu. capacity
//
int GetAccuCapacity(UPS_INFO_FIFO Ups)
{
    float fVal;

    if (IsStatusSet(Ups.ulUpsState, BATTERY_WORK))
	fVal = ((((float)Ups.ulUpsAccuValue[0]-95.0)*100.0)/(124.5-95.0));
    else
	if (!IsStatusSet(Ups.ulUpsState, STANDBY_WORK))
	    fVal = ((((float)Ups.ulUpsAccuValue[0]-95.0)*100.0)/(135.5-95.0));
    if (fVal > 100.0)
	fVal = 100.0;
    if (fVal < 0.0)
	fVal = 0.0;
    fVal *= 10.0;
    return ((int)fVal);
}

//
// Returns Accu. capacity string
//
char *GetAccuCapacityString(char *szBuff, UPS_INFO_FIFO Ups)
{
    float fVal, fMul;

    fMul = 1.0;
    /*
    if ((Ups.uiUpsType != 81) && (Ups.uiUpsType != 82))
    {
        fMul = 2.0;
    }
    */
    *szBuff = 0;
    
    if (IsStatusSet(Ups.ulUpsState, BATTERY_WORK))
	fVal = ((((float)Ups.ulUpsAccuValue[0]-95.0)*100.0)/(124.5-95.0));
    else
	if (!IsStatusSet(Ups.ulUpsState, STANDBY_WORK))
	    fVal = ((((float)Ups.ulUpsAccuValue[0]-95.0)*100.0)/(135.5-95.0));
    if (fVal > 100.0)
	fVal = 100.0;
    if (fVal < 0.0)
	fVal = 0.0;
    
    sprintf(
	szBuff,
	"%.1f",
	fVal);
    return szBuff;
}

//
// Returns warning string
//
char *Warning(char *szBuff, UPS_INFO_FIFO Ups)
{
    *szBuff = 0;
    if (IsStatusSet(Ups.ulUpsState, BATTERY_WORK))
	return ("Zachowaj swoja prace! Wyloguj sie.");
    if ((Ups.ulUpsAccuValue[0] < 105) && 
	IsStatusSet(Ups.ulUpsState, BATTERY_WORK))
	return ("Wysoki stan rozladowania akumulatora!");
    strcpy(szBuff, "Nie mam uwag.");
    return ("Nie mam uwag.");
}

//
// Returns Time (H:M:S) format string
//
char *GetTimeString(char *szBuff, unsigned long ulSec)
{
    int	ih, im, is;
    
    *szBuff = 0;
    ih = (ulSec / 3600);
    im = ((ulSec - (ih * 3600)) / 60);
    is = ((ulSec - (ih * 3600) - (im * 60)));
    sprintf(szBuff, "%02d:%02d:%02d", ih, im, is);
    return (szBuff);
}

void PrintAutotestResult()
{
    printf("Content-type: text/html \n\n");
    printf("<meta http-equiv=\"Refresh\" content=\"20; url=psnet.cgi?op=0\">\n");
    printf("<html>\n");
    printf("<head>\n");
    printf("<title>POWERSOFT STATISTICS PAGE</title>\n");
    printf("</head>\n");
    printf("<body>\n");

    printf(
        "<p align=\"center\"><font face=\"MS Sans Serif\"><big><big><strong>Autotest zostal wykonany poprawnie.</strong></big></big></font></p>\n"
	"<p align=\"center\">&nbsp;</p>\n"
        "<p align=\"center\"><a href=\"psnet.cgi?op=0\"><strong>Odczytaj informacje o zasilaczu</strong></a></p>\n"
	"<p align=\"center\"><a href=\"psnet.cgi?op=1\"><strong>Wykonaj test zasilacza</strong></a></p>\n"
	);
    printf("</body>\n");
    printf("</html>\n");
}

//
// Printing information to Window
//
void PrintUPSInfo(UPS_INFO_FIFO *Ups)
{
    char	szBuffer[STR_LEN];
    printf("Content-type: text/html \n\n");
    printf("<meta http-equiv=\"Refresh\" content=\"15; url=psnet.cgi?op=0\">\n");
    printf("<html>\n");
    printf("<head>\n");
    printf("<title>POWERSOFT - PANEL INFORMACYJNY</title>\n");
    printf("</head>\n");
    printf("<body>\n");

    printf(
        "<p align=\"center\"><font face=\"MS Sans Serif\"><big><big><strong>Witam w programie statystycznym PowerSoft.</strong></big></big></font></p>\n"
	"<p align=\"center\">&nbsp;</p>\n"
        "<p align=\"center\"><a href=\"psnet.cgi?op=0\"><strong>Odczytaj informacje o zasilaczu</strong></a></p>\n"
	"<p align=\"center\"><a href=\"psnet.cgi?op=1\"><strong>Wykonaj test zasilacza</strong></a></p>\n"
	);

    printf(
	"<div align=\"center\"><center>\n"
        "<table border=\"1\" width=\"628\">\n"
	);
    printf(
	"  <caption>Parametry monitorowanego zasilacza</caption>\n"
        "  <tr>\n"
	"    <td width=\"537\"><small>TYP ZASILACZA</small></td>\n"
        "    <td width=\"46\" align=\"center\">&nbsp;</td>\n"
	"    <td width=\"170\"><small>%s</small></td>\n"
        "  </tr>\n",
	GetTypeUpsName(&UpsInfoFifo)
	);
    printf(
	"  <tr>\n"
        "    <td width=\"537\"><small>TRYB PRACY ZASILACZA</small></td>\n"
	"    <td width=\"46\" align=\"center\">&nbsp;</td>\n"
        "    <td width=\"170\"><small>%s</small></td>\n"
	"  </tr>\n",
	GetUpsState(&UpsInfoFifo)
	);
    printf(
        "  <tr>\n"
	"    <td width=\"537\"><small>NAPIECIE WEJSCIOWE</small></td>\n"
        "    <td width=\"46\" align=\"center\"><small>[V]</small></td>\n"
	"    <td width=\"170\"><small>%ld</small></td>\n"
        "  </tr>\n",
	UpsInfoFifo.ulUpsACValue[0]
	);
    printf(
	"  <tr>\n"
        "    <td width=\"537\"><small>ROZNICA WART. CHWILOWEJ NAPIECIA WEJSCIOWEGO</small></td>\n"
	"    <td width=\"46\" align=\"center\"><small>[V]</small></td>\n"
        "    <td width=\"170\"><small>%ld</small></td>\n"
	"  </tr>\n",
	UpsInfoFifo.ulUpsACDifference
	);
    printf(
        "  <tr>\n"
        "    <td width=\"537\"><small>NAPIECIE AKUMULATORA</small></td>\n"
	"    <td width=\"46\" align=\"center\"><small>[V]</small></td>\n"
        "    <td width=\"170\"><small>%s</small></td>\n"
	"  </tr>\n",
	GetAccuString(szBuffer, UpsInfoFifo.ulUpsAccuValue[0])
	);
    printf(
        "  <tr>\n"
	"    <td width=\"537\"><small>POJEMNOSC AKUMULATORA</small></td>\n"
        "    <td width=\"46\" align=\"center\"><small>[%%]</small></td>\n"
	"    <td width=\"170\"><small>%s</small></td>\n"
        "  </tr>\n",
	GetAccuCapacityString(szBuffer, UpsInfoFifo)
	);
    printf(
	"  <tr>\n"
        "    <td width=\"537\"><small>MOC PODLACZONEGO OBCIAZENIA</small></td>\n"
        "    <td width=\"46\" align=\"center\"><small>[VA]</small></td>\n"
	"    <td width=\"170\"><small>%d</small></td>\n"
	"  </tr>\n",
	(int)((float)UpsInfoFifo.ulUpsPowerValue[0] * 1.666)
	);
    printf(
        "  <tr>\n"
	"    <td width=\"537\"><small>CZAS DO ZAMKNIECIA SYSTEMU</small></td>\n"
        "    <td width=\"46\" align=\"center\"><small>[H:M:S]</small></td>\n"
	"    <td width=\"170\"><small>%s</small></td>\n"
        "  </tr>\n",
	GetTimeString(szBuffer, UpsInfoFifo.ulTimeToShutdown1)
	);
    printf(
	"  <tr>\n"
        "    <td width=\"537\"><small>CZAS DO WYKONANIA AUTOTESTU</small></td>\n"
	"    <td width=\"46\" align=\"center\"><small>[H:M:S]</small></td>\n"
        "    <td width=\"170\"><small>%s</small></td>\n"
	"  </tr>\n",
	GetTimeString(szBuffer, UpsInfoFifo.ulTimeToAutotest1)
	);
    printf(
        "  <tr>\n"
	"    <td width=\"537\"><small>UWAGI!</small></td>\n"
        "    <td width=\"46\" align=\"center\">&nbsp;</td>\n"
        "    <td width=\"170\"><small>%s</small></td>\n"
	"  </tr>\n",
	Warning(szBuffer, UpsInfoFifo)
	);
    printf(
	"</table>\n"
        "</center></div>\n"
	);

    printf("</body>\n");
    printf("</html>\n");
}

//
// Terminate program from signal
//
void Terminate(int iSig)
{
    sleep(1);
    _exit(0);
}

//
// Terminate threads
//
void TerminateThread()
{
    RestoreSystemSignals();
    _exit(0);
}

//
// Connection to UPS via FIFO
//
void ConnectToUPS()
{
    char	szwStr[256];
    pid_t	pPid;
    FILE	*fPid;
    int		iError = 0;
    
    // Checking for exists PowerSoft
    fPid = fopen(POWERSOFTPID, "r");
    if (fPid != NULL)
    {
	fread(szwStr, sizeof(char), 16, fPid);
	pPid = atol(szwStr);
        if ((kill(pPid, 0) == -1) && (errno == ESRCH))
	    iError = 1;
	fclose(fPid);
    }
    else
	iError = 1;
    if (iError == 1)
	return;

    if (ReadFifo(&UpsInfoFifo, 0) == SUCCESS_TWOSIDE)
    {
	// This Must be do
	UpsInfoFifo.ulOperation = 0;
	FifoExchange(&UpsInfoFifo, 0);
    }
    PrintUPSInfo(&UpsInfoFifo);
}

//
// Autotesting UPS parameters
//
void Autotest()
{
    char	szwStr[256];
    pid_t	pPid;
    FILE	*fPid;
    int		iError = 0;
    
    // Checking if PowerSoft exists
    fPid = fopen(POWERSOFTPID, "r");
    if (fPid != NULL)
    {
	fread(szwStr, sizeof(char), 16, fPid);
	pPid = atol(szwStr);
        if ((kill(pPid, 0) == -1) && (errno == ESRCH))
	    iError = 1;
	fclose(fPid);
    }
    else
    {
	iError = 1;
    }
    if (iError == 1)
    {
	// "PowerSoft nie jest uruchomiony!";
	return;
    }

    if (ReadFifo(&UpsInfoFifo, 0) == SUCCESS_TWOSIDE)
    {
	//memset(&UpsInfoFifo, 0, sizeof(UPS_INFO_FIFO));
	UpsInfoFifo.ulOperation = FIFO_AUTOTEST;
	FifoExchange(&UpsInfoFifo, 0);
    }
    PrintAutotestResult();
    //PrintUPSInfo(&UpsInfoFifo);
}

//
// Printing error information to log
//
void PrintHackerInfo()
{
    printf("Content-type: text/html \n\n");
    printf("<html>\n");
    printf("<head>\n");
    printf("<title>POWERSOFT - PANEL INFORMACYJNY</title>\n");
    printf("</head>\n");
    printf("<body>\n");

    printf(
        "<p align=\"center\"><font face=\"MS Sans Serif\"><big><big><strong>Blad programu PowerSoft.</strong></big></big></font></p>\n"
	"<p align=\"center\">&nbsp;</p>\n"
	"<p align=\"center\">&nbsp;</p>\n"
	"<p align=\"center\"><font face=\"MS Sans Serif\"><big><big>Bledny parametr skryptu!</big></big></font></p>\n"
	"<p align=\"center\"><font face=\"MS Sans Serif\"><big><big>Program zarejestrowal twoj numer IP (%s)</big></big></font></p>\n",
	getenv("REMOTE_ADDR")
	);
    printf("</body>\n");
    printf("</html>\n");
}

void PrintInfo(char *szBuff)
{
    printf("Content-type: text/html \n\n");
    printf("<html>\n");
    printf("<head>\n");
    printf("<title>POWERSOFT - PANEL INFORMACYJNY</title>\n");
    printf("</head>\n");
    printf("<body>\n");

    printf(
	"<p align=\"center\">%s</p>\n",
	szBuff
	);
    printf("</body>\n");
    printf("</html>\n");
}

//
// Main function
//
int main(int argc, char** argv)
{
    int		iError, i;
    //char	szConfigPath[4096] = "/etc/powersoft/powersoft.conf";

    i = iError = 0;

    InitAllVariables();
    InitSystemSignals();

    i = strlen(getenv("QUERY_STRING"));
    if (i > 0 && i < 5)
    {
	char szEnv[6];
	strcpy(szEnv, getenv("QUERY_STRING"));
	if (strstr(szEnv, "op=") != 0)
	{
	    iOperation = szEnv[3] - 48;
	}
    }

    switch(iOperation)
    {
	case 0: ConnectToUPS();
		break;
	case 1: Autotest();
		break;
	default: PrintHackerInfo();
		break;
    }

    return EXIT_SUCCESS;
}

